/* global nsVmap, oVmap, goog, ol, angular */

/**
 * @author: Armand Bahi
 * @Description: Fichier contenant la classe nsVmap.nsToolsManager.Print
 * cette classe permet l'initialisation des outils de locatlisation
 */
goog.provide('nsVmap.nsToolsManager.Print');
goog.require('oVmap');
goog.require('nsVmap.nsToolsManager.PrintBox');

/**
 * @classdesc
 * Class {@link nsVmap.nsToolsManager.Print}: Add the print tools defined in data/tools.json
 *
 * @constructor
 * @export
 */
nsVmap.nsToolsManager.Print = function () {
    oVmap.log('nsVmap.nsToolsManager.Print');
};
goog.exportProperty(nsVmap.nsToolsManager, 'Print', nsVmap.nsToolsManager.Print);

/**
 * Run the printing process
 * @param {object} opt_options
 * @param {object} opt_options.scope
 * @param {ol.Extent} opt_options.extent
 * @param {string} opt_options.mapId
 * @param {string} opt_options.templateId
 * @param {number} opt_options.resolutionCoeff
 * @param {array<ol.Feature>} opt_options.features feature to zoom on
 * @param {number} opt_options.featuresZoom
 * @returns {undefined}
 * @export
 */
nsVmap.nsToolsManager.Print.prototype.print = function (opt_options) {
    oVmap.log('nsVmap.nsToolsManager.Print.prototype.print');

    var scope = angular.element($("#select-print-format")).scope();
    scope.$evalAsync(function (scope) {
        scope['ctrl'].print(opt_options);
    });
};

/**
 * App-specific directive wrapping the print tools. The directive's
 * controller has a property "map" including a reference to the OpenLayers
 * map.
 *
 * @return {angular.Directive} The directive specs.
 * @constructor
 */
nsVmap.nsToolsManager.Print.prototype.printDirective = function () {
    oVmap.log("nsVmap.nsToolsManager.Print.prototype.printDirective");
    return {
        restrict: 'A',
        scope: {
            'map': '=appMap',
            'lang': '=appLang',
            'currentAction': '=appAction'
        },
        controller: 'AppprintController',
        controllerAs: 'ctrl',
        bindToController: true,
        templateUrl: oVmap['properties']['vmap_folder'] + '/' + 'template/tools/print.html'
    };
};

/**
 * App controler
 * @export
 * @constructor
 * @ngInject
 */
nsVmap.nsToolsManager.Print.prototype.printController = function ($timeout, $compile, $scope, $q) {
    oVmap.log("nsVmap.nsToolsManager.Print.prototype.printController");

    var this_ = this;

    /**
     * @private
     */
    this.$timeout_ = $timeout;

    /**
     * @private
     */
    this.$compile_ = $compile;

    /**
     * @private
     */
    this.$scope_ = $scope;

    /**
     * @private
     */
    this.$q_ = $q;

    /**
     * @public
     */
    $scope['modelIndex'] = 0;

    $scope["managePrintZoneInProgress"] = false;
    $scope["manageComparePrintZoneInProgress"] = false;

    /**
     * @public
     */
    $scope['selectedPrintstyleId'] = "";

    /**
     * Prints models
     */
    this['models'];

    /**
     * Prints styles
     */
    this['printStyles'] = [];

    /**
     * Print selectedModel used
     */
    this['selectedModel'];

    /**
     * Print zone resolution
     */
    this['resolution'] = 2;

    /**
     * dpi selection
     */
    this['dpi'] = 1;

    /**
     * Scale selections
     */
    this['scale'] = 'auto';

    /**
     * The current scale
     */
    this['currentScale'] = oVmap.getMap().getScale({
        'pretty': true
    });

    /**
     * Définit si on utilise l 'étendue de l'outil d'impression
     */
    this['useScaleFromList'] = true;

    /**
     * Définit si on utilise l'étendue de la carte
     */
    this['useMapExtent'] = false;

    /**
     * Scale list
     */
    this['aScales'] = [];

    /**
     * Format d'impression
     */
    this['outputformats_id'] = 'pdf';

    /**
     * Listen the maps moveends to set this.currentScale
     * @private
     */
    this.scaleListener_;

    /**
     * Html definition of the template
     * @private
     */
    this.template_;

    /**
     * Size of the map printZone
     * @private
     */
    this.printedMapSize_;

    /**
     * Size of the compare map printZone
     * @private
     */
    this.printedCompareMapSize_;

    /**
     * Size of the overview map printZone
     * @private
     */
    this.printedOverviewMapSize_;

    /**
     * this.printedMapSize_ resize coeff
     * @private
     */
    this.resizeCoeff_;

    /**
     * The print box object
     * @private
     */
    this.printBox_ = new nsVmap.nsToolsManager.PrintBox({'map_object':oVmap.getMap()});
    /**
     * The print box object
     * @private
     */
    this.printBoxCompare_ = new nsVmap.nsToolsManager.PrintBox({'map_object':oVmap.getMapCompare()});

    /**
     * Is printing in progress
     */
    this.isPrintingInProgress_ = false;

    // mise à jour de l'échelle dans le forulaire d'impression lors d'un mouvement sur la carte
    this.listenScaleChanges();

    $('#print-select-btn').click(function () {
        if ($('#print-select-btn').hasClass('active')) {
            // Charge les paramètres d'impression
            this_.loadPrintProperties($scope['modelIndex']).then(function () {
                if (this_['models'].length > 0) {
                    if (!goog.isDefAndNotNull(this_['models'][$scope['modelIndex']]) || $scope['modelIndex'] === 0) {
                        $scope['modelIndex'] = "0";
                    }
                    this_['printStyles'] = [];
                    this_.loadModelParmas($scope['modelIndex']);
                }
            });

            // Valeur par défaut
            this_['useMapExtent'] = false;
        }
    });

    oVmap['scope'].$on('addScaleOnMap', function () {
        this_.aScales = this_.scalePropertiesToArray();
    }, true);

    // Supprime la zone d'impression lors d'un toggleOutTools
    oVmap['scope'].$on('toggleOutTools', function () {
        if (!$('#print-select-btn').hasClass('active')) {

            // Si l'impression est en cours on ne recharge pas la légende à ce moment là
            if (!this_.isPrintingInProgress_ && !this_.printBox_.isHide()) {
                oVmap.getMapManager().getMapLegendTool().loadLegend();
            }

            this_.printBox_.hide();
            this_.printBoxCompare_.hide();
            $scope["managePrintZoneInProgress"] = false;
            $scope["manageComparePrintZoneInProgress"] = false;
        }
    });

    // Si la taille d'écran change
    $(window).resize(function () {
        // On met à jour la zone d'impression
        this_.printedMapSize_ = this_.getPrintedMapSize(this_.template_, '#map1', '#map_image');
        this_['resizeCoeff_'] = this_.printedMapSize_.resizeCoeff;
        this_['resolution'] = this_.printedMapSize_.resizeCoeff;
        this_.changeScale(this_['scale']);
    });

    // Recharge l'échelle lorsque l'utilisateur change useScaleFromList
    $scope.$watch('ctrl.useScaleFromList', function() {
        if (this_['useMapExtent'] !== true) {
            this_.changeScale(this_['scale']);
        }
        // Recharge la légende quand on coche ou décoche la case échelle
        // On attend 100ms pour que la printbox ai eu le temps de se mettre à jour
        setTimeout(function() {
            this_.reloadLegend();
        }, 100);
    });
};

/**
 * Load the print models
 * @returns {undefined}
 * @private
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.loadPrintProperties = function (modelIndex) {
    oVmap.log('nsVmap.nsToolsManager.Print.printController.loadPrintProperties');

    var this_ = this;
    var deferred = this.$q_.defer();

    // Coche la case d'échelle à l'ouverure d'outils d'impression
    this_['useScaleFromList'] = true;

    ajaxRequest({
        'method': 'GET',
        'url': oVmap['properties']['api_url'] + '/vmap/userprinttemplates',
        'headers': {
            'Accept': 'application/json'
        },
        'params': {
            'distinct': true,
            'order_by': oVmap['properties']['vmap']['print_template_list_order'] ? 'name' : 'printtemplate_order|name',
            'attributs': 'rt_format_id|name|orientation|printstyles|printstyle_id|printtemplate_id|printtemplate_order|variables'
        },
        'scope': this.$scope_,
        'success': function (response) {

            if (goog.isDefAndNotNull(this_['models'])) {
                var oldModels = angular.copy(this_['models']);
            }
            this_['models'] = [];

            // Vérifie si le fichier a bien été chargé, donne un modèle par défaut en cas d'erreur
            if (goog.isDefAndNotNull(response['data'])) {
                if (goog.isArray(response['data']['userprinttemplates'])) {
                    if (response['data']['userprinttemplates'].length > 0) {
                        this_['models'] = response['data']['userprinttemplates'];
                        if (!goog.isDefAndNotNull(oldModels)) {
                            this_.$scope_['selectedPrintstyleId'] = null;
                        }
                    }
                }
            }

            // Recharge la légende pour éviter les erreurs d'association de blob
            // oVmap.getMapManager().getMapLegendTool().loadLegend();

            deferred.resolve();
        }
    });
    return deferred.promise;
};

/**
 * Listen for scale changes and update the scale on the print form
 * @returns {undefined}
 * @private
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.listenScaleChanges = function () {
    oVmap.log('nsVmap.nsToolsManager.Print.printController.listenScaleChanges');

    // Vérifie si la résolution (niveau de zoom) change
    var i = 0;
    var this_ = this;
    this.scaleListener_ = this['map'].on('moveend', function () {
        i++;
        var ii = angular.copy(i);
        setTimeout(function () {
            if (i === ii) {

                if (goog.isDefAndNotNull(this_.printBox_) && $('#print-select-btn').hasClass('active')) {
                    // Recharge la légende à chaque movement sur la carte
                    this_.reloadLegend();
                }

                if (this_['currentAction'] !== 'print-modifyPrintZone') {
                    this_['scale'] = 'auto';
                    this_['currentScale'] = oVmap.getMap().getPrettyScale(oVmap.getMap().getScale() / this_.resizeCoeff_);
                    this_.$scope_.$apply();
                }
            }
        }, 100);
    });
};

/**
 * Unlisten for scale changes and stop updating the scale on the print form
 * @returns {undefined}
 * @private
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.unlistenScaleChanges = function () {
    oVmap.log('nsVmap.nsToolsManager.Print.printController.unlistenScaleChanges');
    ol.Observable.unByKey(this.scaleListener_);
};

/**
 * Charge le template d'un modèle et l'écrase dans this.models
 *
 * @param  {string} sPrinttemplateId
 * @return {promise}
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.loadModelTemplate = function (sPrinttemplateId) {
    oVmap.log('nsVmap.nsToolsManager.Print.printController.loadModelTemplate');

    var this_ = this;
    var deferred = this.$q_.defer();

    ajaxRequest({
        'method': 'GET',
        'url': oVmap['properties']['api_url'] + '/vmap/userprinttemplates/' + sPrinttemplateId,
        'headers': {
            'Accept': 'application/json'
        },
        'params': {
            'skip_style_definition': true
        },
        'scope': this.$scope_,
        'success': function (response) {

            if (goog.isDefAndNotNull(response['data'])) {
                if (goog.isDefAndNotNull(response['data']['userprinttemplates'])) {
                    if (goog.isDefAndNotNull(response['data']['userprinttemplates'][0])) {
                        if (goog.isDefAndNotNull(response['data']['userprinttemplates'][0]['definition']) &&
                            goog.isDefAndNotNull(response['data']['userprinttemplates'][0]['printtemplate_id'])) {
                            if (response['data']['userprinttemplates'][0]['printtemplate_id'] === sPrinttemplateId) {
                                for (var i = 0; i < this_['models'].length; i++) {
                                    if (this_['models'][i]['printtemplate_id'] === sPrinttemplateId) {
                                        this_['models'][i]['definition'] = response['data']['userprinttemplates'][0]['definition'];
                                    }
                                }
                            }
                        }
                    }
                }
            }

            deferred.resolve();
        }
    });
    return deferred.promise;
}

/**
 * Load the selectedModel params
 * @param {number} modelIndex index in models
 * @export
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.loadModelParmas = function (modelIndex) {
    oVmap.log('nsVmap.nsToolsManager.Print.printController.loadModelParmas');

    // Ancien modèle d'impression
    var oldModel = angular.copy(this['selectedModel']);

    // Charge le modèle
    this['selectedModel'] = this['models'][modelIndex];
    
    // Si le format est A0 on recharge la carte par défaut car la légende n'est pas dynamique en A0
    if (this['selectedModel']['format'] === 'A0') {
        oVmap.getMapManager().getMapLegendTool().loadLegend('');
    }
    // Charge le template
    this.setTemplate(this['selectedModel'], this.loadModelParmas2);

    // Charge la liste des styles disponibles
    this['printStyles'] = this['selectedModel']['printstyles'];

    // Nouveau modèle d'impression
    var newModel = this['models'][modelIndex];

    // Récupère les variables de l'ancien modèle sur le nouveau
    if (goog.isDefAndNotNull(oldModel) && goog.isDefAndNotNull(newModel)) {
        if (goog.isArray(oldModel['variables']) && goog.isArray(newModel['variables'])) {
            
            // Double boucle sur les variables des deux modèles
            for (var i = 0; i < newModel['variables'].length; i++) {
                for (var ii = 0; ii < oldModel['variables'].length; ii++) {
                    
                    // si la variable est présente dans les deux modèles
                    if (newModel['variables'][i]['name'] === oldModel['variables'][ii]['name']) {
                        newModel['variables'][i]['value'] = oldModel['variables'][ii]['value'];
                    }
                }
            }
        }
    }
};

/**
 * Load the selectedModel params 2
 * @private
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.loadModelParmas2 = function () {
    oVmap.log('nsVmap.nsToolsManager.Print.printController.loadModelParmas2');

    // Initialise la liste des échelles
    this.aScales = this.scalePropertiesToArray();
    
    // Récupère la taille de la carte this.printedMapSize_ ainsi que le coefficient this.resizeCoeff_
    this.printedMapSize_ = this.getPrintedMapSize(this.template_, '#map1', '#map_image');
    this.printedOverviewMapSize_ = this.getPrintedMapSize(this.template_, '#map1', '#map_overview');
    
    this.resizeCoeff_ = this.printedMapSize_.resizeCoeff;
    
    // Pré-rempli le champ "Résolution"
    this['resolution'] = this.resizeCoeff_;

    // Ajuste l'échelle avec le niveau de détail
    this['currentScale'] = oVmap.getMap().getPrettyScale(oVmap.getMap().getScale() / this.resizeCoeff_);
    
    // Dessine un carré d'impression à chaque mouvement de la carte
    this.printBox_.setSize(this.printedMapSize_.printedMapSize);
    this.printBox_.show();


    if(this.$scope_["$root"]["compare_enabled"]){
        this.printedCompareMapSize_ = this.getPrintedMapSize(this.template_, "#map1Compare", "#map_image_compare");

        if(this.printedCompareMapSize_ !== 0){
            // Pré-rempli le champ "Résolution"
            this['resolutionCompare'] = this.printedCompareMapSize_.resizeCoeff;

            // Ajuste l'échelle avec le niveau de détail
            this['currentScaleCompare'] = oVmap.getMapCompare().getPrettyScale(oVmap.getMapCompare().getScale() / this.printedCompareMapSize_.resizeCoeff);
            
            // Dessine un carré d'impression à chaque mouvement de la carte
            this.printBoxCompare_.setSize(this.printedCompareMapSize_.printedMapSize);
            this.printBoxCompare_.show();
        } else {
            this.printBoxCompare_.hide();
        }
    }

    if (goog.isDefAndNotNull(this['selectedModel']) && this['selectedModel']['format'] === 'A0') {
        oVmap.getMapManager().getMapLegendTool().loadLegend('');
    } else {
        this.reloadLegend();
    }

    // Supprime le template du body de Vmap
    $(this.template_).remove();
};

/**
 * Set the template in this.template_ and run loadModelParmas2
 * @param {object} oModel
 * @param {string|undefined} oModel.url template url starting form the vmap folder
 * @param {function} callback
 * @returns {undefined}
 * @private
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.setTemplate = function (oModel, callback) {
    oVmap.log('nsVmap.nsToolsManager.Print.printController.setTemplate');

    var this_ = this;

    if (!goog.isDefAndNotNull(oModel['definition']) && !goog.isDefAndNotNull(oModel['url'])) {
        this.loadModelTemplate(oModel['printtemplate_id']).then(function() {
            if (!goog.isDefAndNotNull(oModel['definition']) && !goog.isDefAndNotNull(oModel['url'])) {
                bootbox.alert('<h4>Aucune définition présente dans le modèle d\'impression</h4>');
            } else {
                this_.setTemplate(oModel, callback);
            }
        });
    }

    this.$scope_['compare_mode'] = this.$scope_["$root"]["compare_enabled"];

    if (goog.isDefAndNotNull(oModel['definition'])) {
        var template = document.createElement("div");
        template.innerHTML = oModel['definition'];

        this.$compile_(template)(this.$scope_);
        this_.template_ = template;

        if (goog.isDefAndNotNull(callback)) {
            setTimeout(function () {
                callback.call(this_, [template]);
            });
        }
    }
};

/**
 * Get the print map size and resolution coeff
 *
 * @param  {type} template
 * @param  {type} sElementMapId
 * @param  {type} sPrintId
 * @return {object}
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.getPrintedMapSize = function (template, sElementMapId, sPrintId) {
    oVmap.log('nsVmap.nsToolsManager.Print.printController.getPrintedMapSize');
    sPrintId = (goog.isDefAndNotNull(sPrintId)) ? sPrintId : '#map_image';
    sElementMapId = (goog.isDefAndNotNull(sElementMapId)) ? sElementMapId : '#map1';

    var imageDiv = $(template).find(sPrintId);

    // Vérifie la présence de '#map_image'
    if (!goog.isDef(imageDiv.get(0))) {
        console.error('Aucune balise ' + sPrintId + ' trouvée dans le template');
        return 0;
    }

    oPrintScope = angular.element($(template).find(sPrintId)).scope();

    // Ajoute temporairement le template au body de Vmap
    $(template).addClass('print_template');
    $('body').append(template);

    var mapHeight = imageDiv.height();
    var mapWidth = imageDiv.width();

    // Vérifie si la taille de la carte est suppérieur à la taille de l'écran
    var resizeCoeff = 1;
    var tmpWidth = angular.copy(mapWidth);
    var tmpHeight = angular.copy(mapHeight);
    while (mapWidth > ($(sElementMapId).width()) || mapHeight > ($(sElementMapId).height())) {
        resizeCoeff++;
        mapWidth = tmpWidth / resizeCoeff;
        mapHeight = tmpHeight / resizeCoeff;
    }

    setTimeout(function () {
        $(template).remove();
    });

    return {
        printedMapSize: [mapWidth, mapHeight],
        realSize: [imageDiv.width(), imageDiv.height()],
        resizeCoeff: resizeCoeff
    }
};

/**
 * Change the map scale for the print-zone
 * @param {number} scale
 * @returns {undefined}
 * @export
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.changeScale = function (scale) {
    oVmap.log('nsVmap.nsToolsManager.Print.prototype.printController.prototype.changeScale');

    // Par defaut la taille du cadre est divisée par deux
    this['currentScale'] = oVmap.getMap().getPrettyScale(oVmap.getMap().getScale() / this.resizeCoeff_);

    // Ce qui signifie que l'échelle aussi
    var viewScale = scale * this.resizeCoeff_;

    // Variable qui stocke la taille du cadre d'impression   
    var boxSize = {};
    if (goog.isDefAndNotNull(this.printedMapSize_)) {
        if (goog.isDefAndNotNull(this.printedMapSize_.printedMapSize)) {
            boxSize[0] = this.printedMapSize_.printedMapSize[0];
            boxSize[1] = this.printedMapSize_.printedMapSize[1];

            // Si l'utilisateur choisi de conserver la taille du cadre d'impression en décochant la checkbox
            if (this.resizeCoeff_ > 1 && !this['useScaleFromList']) {
                viewScale = scale;
                boxSize[0] = this.printedMapSize_.printedMapSize[0] * this.resizeCoeff_;
                boxSize[1] = this.printedMapSize_.printedMapSize[1] * this.resizeCoeff_;
            }
            this.printBox_.setSize(boxSize);
            this.printBoxCompare_.setSize(boxSize);
        }
    }

    if (scale === 'auto')
        return 0;

    // Positionne la vue en fonction de la printZone
    if (this['currentAction'] === 'print-modifyPrintZone') {
        this.printBox_.setScale(scale);
        this.currentScale = scale;
        return 0;
    }

    // Change la résolution en fonction de l'échelle
    oVmap.getMap().setScale(viewScale);

    // Change l'échelle de la carte comparée si besoin
    if(this.$scope_["$root"]["compare_enabled"]) {
        setTimeout(function () {
            oVmap.getMapCompare().setScale(oVmap.getMap().getScale());
        });
    }
};

/**
 * Detach the printZone from the view and allow to translate it
 * @export
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.managePrintZone = function () {
    oVmap.log('nsVmap.nsToolsManager.Print.prototype.printController.prototype.managePrintZone');

    if (this['currentAction'] === 'print-modifyPrintZone' && this.$scope_['managePrintZoneInProgress']) {
        this.printBox_.unmanagePrintBox();
        this.$scope_["managePrintZoneInProgress"] = false;

        // Recharge la légende quand on désactive l'action pour géré la zone d'impression
        this.reloadLegend();
    } else {
        this.$scope_["managePrintZoneInProgress"] = true;
        this.printBox_.managePrintBox('print-modifyPrintZone', this['selectedModel']['format'], false);
    }
};

/**
 * Detach the printZone from the view and allow to translate it
 * @export
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.manageComparePrintZone = function () {
    oVmap.log('nsVmap.nsToolsManager.Print.prototype.printController.prototype.manageComparePrintZone');

    if (this['currentAction'] === 'print-modifyPrintZone' && this.$scope_["manageComparePrintZoneInProgress"]) {
        this.printBoxCompare_.unmanagePrintBox();
        this.$scope_["manageComparePrintZoneInProgress"] = false;

        // Recharge la légende quand on désactive l'action pour géré la zone d'impression
        this.reloadLegend();
    } else {
        //this.printBox_.unmanagePrintBox();
        this.$scope_["manageComparePrintZoneInProgress"] = true;
        this.printBoxCompare_.managePrintBox('print-modifyPrintZone', this['selectedModel']['format'], true);
    }
};

/**
 * Utilise l'etendue de la carte
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.attachPrintZoneToMapExtent = function () {
    oVmap.log('nsVmap.nsToolsManager.Print.prototype.printController.prototype.attachPrintZoneToMapExtent');

    // Toogle this['useMapExtent'] value
    if (!this['useMapExtent']) {
        this['useMapExtent'] = true;
        this['useScaleFromList'] = false;
    } else {
        this['useMapExtent'] = false;
        this['useScaleFromList'] = true;
    }

    // Atache / relache la printbox à l'étendue de la carte
    if (this['useMapExtent']) {
        this.printBox_.attachToMapExtent();
        if (this.$scope_['$root']["compare_enabled"]) {
            this.printBoxCompare_.attachToMapExtent();
        }
    } else {
        this.printBox_.unattachToMapExtent();
        if (this.$scope_['$root']["compare_enabled"]) {
            this.printBoxCompare_.unattachToMapExtent();
        }
    }

    // Recharge la légende après avoir attaché / relaché la printbox à l'étendue de la carte
    this.reloadLegend();
}

/**
 * Prepare the print params and launch it
 * @returns {undefined}
 * @export
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.prepareAndLaunchPrint = function () {
    oVmap.log('nsVmap.nsToolsManager.Print.prototype.printController.prototype.prepareAndLaunchPrint');

    // Enregistre l'échelle actuelle
    this.currentScale_ = oVmap.getMap().getScale({
        'pretty': true
    });

    // Prépare scope
    var scope = {};
    for (var i = 0; i < this['selectedModel']['variables'].length; i++)
        scope[this['selectedModel']['variables'][i]['name']] = this['selectedModel']['variables'][i]['value'];

    // Prépare extent
    var extent = this.printBox_.getExtent();

    // templateId
    var templateId = this['selectedModel']['printtemplate_id'];

    var aLocationOverlayFeatures = oVmap.getMap().getLocationOverlayFeatures().getArray();
    var aPopupOverlayFeatures = oVmap.getMap().getPopupOverlayFeatures().getArray();
    var aSelectionOverlayFeatures = oVmap.getMap().getSelectionOverlayFeatures().getArray();
    var aMeasureFeatures = oVmap.getMapManager().getMeasureFeatures();

    var oPrintOptions = {
        scope: scope,
        extent: extent,
        templateId: templateId,
        printStyleId: this.$scope_['selectedPrintstyleId'],
        resolutionCoeff: this['dpi'],
        outputformats_id: this['outputformats_id'],
        measureFeatures: aMeasureFeatures,
        features: goog.array.concat(aLocationOverlayFeatures, aPopupOverlayFeatures, aSelectionOverlayFeatures)
    };

    // récupération des infos pour la carte de comparaison si le mode comparaison est actif
    if(this.$scope_['$root']["compare_enabled"]){
        if(this.printedCompareMapSize_ !== 0){
            // Enregistre l'échelle actuelle
            this.currentScaleCompare_ = oVmap.getMapCompare().getScale({
                'pretty': true
            });
            oPrintOptions.extentCompare = this.printBoxCompare_.getExtent();
        }
    }

    // Lance l'impression
    var returnPrint = this.print(oPrintOptions);

    if (returnPrint === 1) {
        // Ferme l'outil d'impression
        oVmap.getToolsManager().getBasicTools().toggleOutTools();
    }
};

/**
 * Run the printing process
 * @param {object} opt_options
 * @param {object|undefined} opt_options.scope
 * @param {ol.Extent|undefined} opt_options.extent
 * @param {string|undefined} opt_options.mapId
 * @param {string} opt_options.templateId
 * @param {string} opt_options.printStyleId
 * @param {number|undefined} opt_options.resolutionCoeff
 * @param {array<ol.Feature>|undefined} opt_options.features feature to zoom on
 * @param {number|undefined} opt_options.featuresZoom
 * @param {boolean|undefined} opt_options.recalcSize true pour recalculer la taille au moment de l'impression (utilisé pour module cadastre)
 * @returns {number} 1 if all semms good
 * @export
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.print = function (opt_options) {
    oVmap.log('nsVmap.nsToolsManager.Print.prototype.printController.prototype.print');

    var this_ = this;

    this.isPrintingInProgress_ = true;

    if (!goog.isDefAndNotNull(opt_options)) {
        console.error('opt_options.templateId not defined');
        return 0;
    }

    // Récupère la date
    var today = new Date();
    var dd = today.getDate();
    var mm = today.getMonth() + 1; //January is 0!
    var yyyy = today.getFullYear();
    if (dd < 10) {
        dd = '0' + dd;
    }
    if (mm < 10) {
        mm = '0' + mm;
    }
    today = dd + '/' + mm + '/' + yyyy;

    // Valeurs par défaut
    var templateId = opt_options.templateId;
    var printStyleId = opt_options.printStyleId;

    // Set scope
    var oPrintScope = goog.isDefAndNotNull(opt_options.scope) ? opt_options.scope : {};
    oPrintScope['date'] = goog.isDefAndNotNull(oPrintScope['date']) ? oPrintScope['date'] : today;
    oPrintScope['compare_mode'] = this.$scope_["$root"]["compare_enabled"] === true ? true : false;
    var sScope = JSON.stringify(oPrintScope);

    // Ouvre la fenêtre d'impression
    var bootboxLoading = bootbox.dialog({
        message: '<div class="text-center mb-0">Impression en cours...</div>',
        closeButton: false
    });

    // Récupère les infos du template
    ajaxRequest({
        'method': 'GET',
        'url': oVmap['properties']['api_url'] + '/vmap/printtemplates/' + templateId,
        'headers': {
            'Accept': 'application/x-vm-json'
        },
        'scope': this.$scope_,
        'success': function (response) {
            var bError = false;
            if (!goog.isDefAndNotNull(response['data'])) {
                bError = true;
            } else if (!goog.isDefAndNotNull(response['data']['data'])) {
                bError = true;
            } else if (!goog.isDefAndNotNull(response['data']['data'][0])) {
                bError = true;
            } else if (!goog.isDefAndNotNull(response['data']['data'][0]['definition'])) {
                bError = true;
            } else if (!goog.isDefAndNotNull(response['data']['data'][0]['rt_format_id'])) {
                bError = true;
            } else if (!goog.isDefAndNotNull(response['data']['data'][0]['rt_orientation_id'])) {
                bError = true;
            }
            if (bError) {
                $.notify('Erreur lors du chargement des données du template ' + templateId, 'error');
                console.error("response: ", response);
                return 0;
            }

            var sFormat = response['data']['data'][0]['rt_format_id'];
            var sOrientation = response['data']['data'][0]['rt_orientation_id'];

            // Cas d'utilisation de la fonction print sans passer par le forumlaire
            if (opt_options.recalcSize === true) {
                this_.setTemplate({
                    'definition': response['data']['data'][0]['definition']
                });
                this_.printedMapSize_ = this_.getPrintedMapSize(this_.template_, '#map1', '#map_image');
                this_.printedOverviewMapSize_ = this_.getPrintedMapSize(this_.template_, '#map1', '#map_overview');
            }

            var mapImageSize = this_.printedMapSize_.realSize;
            var overviewSize = this_.printedOverviewMapSize_.realSize;

            var oJsonDefOptions = {
                mapId: opt_options.mapId,
                resolutionCoeff: opt_options.resolutionCoeff,
                extent: opt_options.extent,
                features: opt_options.features,
                featuresZoom: opt_options.featuresZoom,
                measureFeatures: opt_options.measureFeatures,
                mapImageSize: mapImageSize,
                overviewSize: overviewSize
            }

            if(goog.isDefAndNotNull(opt_options.extentCompare)){
              oJsonDefOptions.extentCompare = opt_options.extentCompare;
              oJsonDefOptions.mapIdCompare = opt_options.mapIdCompare;
              oJsonDefOptions.mapImageSizeCompare = this_.printedCompareMapSize_.realSize;
            }

            var mapsJson = this_.getMapsJsonDef(oJsonDefOptions);

            // Récupère includesJson
            this_.getIncludes(opt_options).then(function(includesJson) {

                // Récupère les infos de l'utilisateur
                this_.getUserInfos_().then(function (oUserInfos) {
    
                    // Merge les infos utilisateur
                    var oPrintScope = JSON.parse(sScope);
                    for (var key in oUserInfos) {
                        if (!goog.isDef(oPrintScope['user_' + key])) {
                            oPrintScope['user_' + key] = oUserInfos[key];
                        }
                    }
    
                    // Sources des couches
                    oPrintScope['layer_sources'] = oVmap.getMapManager().getMapSources();
                    oPrintScope['layer_sources_compare'] = oVmap.getMapManager().getMapSources(true);
    
                    sScope = JSON.stringify(oPrintScope);
    
                    ajaxRequest({
                        'method': 'POST',
                        'url': oVmap['properties']['api_url'] + '/vmap/printtemplateservices',
                        'headers': {
                            'Accept': 'application/x-vm-json'
                        },
                        'data': {
                            'printtemplate_id': templateId,
                            'printstyle_id': printStyleId,
                            'format': sFormat,
                            'orientation': sOrientation,
                            'includes_json': includesJson,
                            'maps_json': mapsJson,
                            'outputformats_id': opt_options['outputformats_id'],
                            'scope_json': sScope
                        },
                        'scope': this_.$scope_,
                        'timeout': 120000,
                        'success': function (response) {
    
                            var bError = false;
                            if (!goog.isDefAndNotNull(response['data'])) {
                                bError = true;
                            } else if (!goog.isDefAndNotNull(response['data']['printtemplateservices'])) {
                                bError = true;
                            } else if (!goog.isDefAndNotNull(response['data']['printtemplateservices']['image'])) {
                                bError = true;
                            } else if (response['data']['status'] !== 1) {
                                bError = true;
                            }
                            if (bError) {
                                $.notify('Une erreur est survenue lors de l\'impression', 'error');
                                this_.printingInProgressEnd(bootboxLoading);
                                console.error("response: ", response);
                                return 0;
                            }
    
                            $.notify('Impression réussie', 'success');
                            this_.printingInProgressEnd(bootboxLoading);
                            oVmap.downloadFileFromUrl(response['data']['printtemplateservices']['image']);
                            delete this_.printedMapSize_;
                            delete this_.printedOverviewMapSize_;
                        },
                        'error': function (response) {
                            console.error(response);
                            $.notify('Une erreur est survenue lors de l\'impression', 'error');
                            this_.printingInProgressEnd(bootboxLoading);
                        }
                    });
                });
            });
        },
        'error': function (response) {
            console.error(response);
            $.notify('Une erreur est survenue lors de l\'impression', 'error');
            this_.printingInProgressEnd(bootboxLoading);
        }
    });
    return 1;
};

/**
 * A la fin d'une impression qu'elle soit réussi ou en erreur
 * @param  {any} bootboxLoading
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.printingInProgressEnd = function (bootboxLoading) {
    // ferme le message d'attente
    bootboxLoading.modal('hide');
    // L'impression n'est plus en cours
    this.isPrintingInProgress_ = false;
    // Recharge la légende pour la carte
    oVmap.getMapManager().getMapLegendTool().loadLegend();
}

/**
 * Get the maps JSON defs
 * @param  {object} opt_options
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.getMapsJsonDef = function (opt_options) {
    oVmap.log('nsVmap.nsToolsManager.Print.prototype.printController.prototype.getMapsJsonDef');

    // Resolution
    var resolutionCoeff = goog.isDefAndNotNull(opt_options.resolutionCoeff) ? opt_options.resolutionCoeff : 1;

    // mapId/mapJson
    var sMapId;
    var sMapJSON;
    var sMapIdCompare;
    var sMapJSONCompare;
    if (goog.isDefAndNotNull(opt_options.mapId)) {
        sMapId = opt_options.mapId;
    } else {
        sMapJSON = oVmap.getMapManager().getJSONLayersTree();
    }
    if (goog.isDefAndNotNull(opt_options.mapIdCompare)) {
        sMapIdCompare = opt_options.mapIdCompare;
    } else if (this.$scope_['$root']["compare_enabled"]) {
        sMapJSONCompare = oVmap.getMapManager().getJSONLayersTree(true);
    }

    // Extent
    var aExtent = goog.isDefAndNotNull(opt_options.extent) ? opt_options.extent : [];
    var sExtent = '';
    var aOverviewExtent = [
        aExtent[0] - (aExtent[2] - aExtent[0]),
        aExtent[1] - (aExtent[3] - aExtent[1]),
        aExtent[2] + (aExtent[2] - aExtent[0]),
        aExtent[3] + (aExtent[3] - aExtent[1])
    ];
    var sOverviewExtent = '';
    var sCompareExtent = '';
    for (var i = 0; i < aExtent.length; i++) {
        if (i > 0) {
            sExtent += '|';
            sOverviewExtent += '|';
            sCompareExtent += (goog.isDefAndNotNull(opt_options.extentCompare)) ? '|' : '';
        }
        sExtent += aExtent[i];
        sOverviewExtent += aOverviewExtent[i];
        sCompareExtent += (goog.isDefAndNotNull(opt_options.extentCompare)) ? opt_options.extentCompare[i] : '';
    }

    // Features
    var aFeatures = goog.isDefAndNotNull(opt_options.features) ? opt_options.features : [];
    var sEWKTFeatures = '';
    for (var i = 0; i < aFeatures.length; i++) {
        if (i > 0)
            sEWKTFeatures += '|';
        sEWKTFeatures += oVmap.getEWKTFromGeom(aFeatures[i].getGeometry());
    }

    var sFeaturesZoom = 100;
    if (goog.isDefAndNotNull(opt_options.featuresZoom)) {
        sFeaturesZoom = opt_options.featuresZoom;
    } else if (goog.isDefAndNotNull(oVmap['properties']['print']['features_zoom'])) {
        sFeaturesZoom = oVmap['properties']['print']['features_zoom'];
    }

    // Measures
    var oMeasureFeatures = null;
    if (goog.isArray(opt_options.measureFeatures)) {
        if (opt_options.measureFeatures.length > 0) {
            oMeasureFeatures = ol.getGeoJSONFromFeatures(opt_options.measureFeatures,
                oVmap.getMap().getOLMap().getView().getProjection().getCode());
        }
    }

    if (goog.isDefAndNotNull(opt_options.mapImageSize)) {
        var imageSize = Math.round(opt_options.mapImageSize[0] * resolutionCoeff) + '|' + Math.round(opt_options.mapImageSize[1] * resolutionCoeff);
        var oMapDef = {
            'map_id': sMapId,
            'map_json': sMapJSON,
            'image_size': imageSize,
            'resolution_coeff': resolutionCoeff,
            'extent': sExtent,
            'features': sEWKTFeatures,
            'features_zoom': sFeaturesZoom
        };
        if (goog.isDefAndNotNull(oMeasureFeatures)) {
            oMapDef['measure_features'] = oMeasureFeatures;
        }
    }

    if (goog.isDefAndNotNull(opt_options.overviewSize)) {
        var oOverviewDef = {
            'map_id': sMapId,
            'map_json': sMapJSON,
            'image_size': Math.round(opt_options.overviewSize[0] * resolutionCoeff) + '|' + Math.round(opt_options.overviewSize[1] * resolutionCoeff),
            'resolution_coeff': resolutionCoeff,
            'extent': sOverviewExtent,
            'features': sEWKTFeatures,
            'features_zoom': 400
        };
    }
    if (goog.isDefAndNotNull(opt_options.mapImageSizeCompare) && this.$scope_.$root["compare_enabled"]) {
        var oCompareDef = {
            'map_id': sMapIdCompare,
            'map_json': sMapJSONCompare,
            'image_size': Math.round(opt_options.mapImageSizeCompare[0] * resolutionCoeff) + '|' + Math.round(opt_options.mapImageSizeCompare[1] * resolutionCoeff),
            'resolution_coeff': resolutionCoeff,
            'extent': sCompareExtent
        };
    }

    var oMapsJson = [];
    if (goog.isDefAndNotNull(oMapDef)) {
        oMapsJson.push({
            'target': '#map_image',
            'map_definition': oMapDef
        });
    }
    if (goog.isDefAndNotNull(oOverviewDef)) {
        oMapsJson.push({
            'target': '#map_overview',
            'map_definition': oOverviewDef
        });
    }

    if (goog.isDefAndNotNull(oCompareDef)) {
        oMapsJson.push({
            'target': '#map_image_compare',
            'map_definition': oCompareDef
        });
    }

    var mapsJson = JSON.stringify(oMapsJson);

    return mapsJson;
}

/**
 * Get Includes
 * @returns {String}
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.getIncludes = function (opt_options) {

    var deferred = this.$q_.defer();
    var this_ = this

    this_.prepareAndGetLegendTemplate(opt_options, '#maplegend').then(function(template) {
        var aIncludes = [{
            'target': '#map_legend',
            'html': template
        }];

        if (this_.$scope_["$root"]["compare_enabled"]) {

            this_.prepareAndGetLegendTemplate(opt_options, '#compare_maplegend').then(function(template) {
                aIncludes.push({
                    'target': '#map_legend_compare',
                    'html': template
                });

                deferred.resolve(JSON.stringify(aIncludes));
            });

        } else {
            deferred.resolve(JSON.stringify(aIncludes));
        }
    });



    return deferred.promise;

};

/**
 * Get the html definition of the template with base64 images
 * @param  {string} sTarget
 * @param {array} dataMapImage
 * @param {string} sFormat
 *
 * @returns {String}
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.prepareAndGetLegendTemplate = function (opt_options, sTarget) {

    var deferred = this.$q_.defer();
    var this_ = this;

    // Si la légende est déjà chargé
    if (oVmap.getMapManager().getMapLegendTool().isLoadedLegend() ||
        (goog.isDefAndNotNull(opt_options.cadastre) && opt_options.cadastre === true)) {
        sTemplate = this_.getLegendTemplate(sTarget);
        deferred.resolve(sTemplate);
    } else { // Sinon on attends un evenement
        var deregisterRunOnceHandler = oVmap['scope'].$on('printMap', function () {
            setTimeout(function() {
                sTemplate = this_.getLegendTemplate(sTarget);

                // Supprime l'evenement car on veut qu'il ne soit exécuté qu'une seule fois
                deregisterRunOnceHandler();
                deregisterRunOnceHandler = null;

                // Renvoi le résultat
                deferred.resolve(sTemplate);
            }, 100);
        });
    }

    return deferred.promise;
};

/**
 * Get the html definition of the template with base64 images
 * @param  {string} sTarget
 * @returns {String}
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.getLegendTemplate = function (sTarget) {

    var template = $(sTarget).clone();

    // Transforme les images en base64
    template.find('img').each(function (index) {

        try {

            var image = this;
            var imgCanvas = document.createElement("canvas");
            var imgContext = imgCanvas.getContext("2d");

            imgCanvas.width = image.width;
            imgCanvas.height = image.height;

            imgContext.drawImage(image, 0, 0, image.width, image.height);

            var base64Img = imgCanvas.toDataURL("image/png");

            $(this).attr('src', base64Img);
            $(this).removeAttr('ng-src');

        } catch (e) {

        }
    });

    var sTemplate = template.html();

    sTemplate = sTemplate.replace(/&amp;/g, '&');
    sTemplate = sTemplate.replace(/"/g, '\\"');

    return sTemplate;
};

/**
 * Get the user infos in a promise
 * @returns {defer.promise}
 * @private
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.getUserInfos_ = function () {
    oVmap.log("nsVmap.nsToolsManager.Print.prototype.printController.prototype.getUserInfos_");

    var deferred = this.$q_.defer();

    ajaxRequest({
        'method': 'GET',
        'url': oVmap['properties']['api_url'] + '/vitis/users/' + sessionStorage['user_id'],
        'headers': {
            'Accept': 'application/x-vm-json'
        },
        'scope': this.$scope_,
        'success': function (response) {
            if (goog.isDefAndNotNull(response['data'])) {
                if (goog.isDefAndNotNull(response['data']['data'])) {
                    if (goog.isDefAndNotNull(response['data']['data'][0])) {
                        deferred.resolve(response['data']['data'][0]);
                    }
                }
            }
        }
    });
    return deferred.promise;
};

/**
 * Return formated number
 * @param {number} scale
 * @returns {undefined}
 * @export
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.formatScale = function (scale, format) {
    oVmap.log('nsVmap.nsToolsManager.Print.prototype.printController.prototype.formatScale');

    return new Intl.NumberFormat(format).format(scale);
};

/**
 * Return formated number
 * @param {number} scale
 * @returns {undefined}
 * @export
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.scalePropertiesToArray = function (scale, format) {
    oVmap.log('nsVmap.nsToolsManager.Print.prototype.printController.prototype.scalePropertiesToArray');

    var aScales = [];
    var sScales = oVmap['properties']['controls']['scales'];
    if (goog.isDefAndNotNull(sScales)) {
        sScales = sScales.toString();
        aScales = sScales.indexOf('|') == -1 ? [sScales] : sScales.split('|');
    }
    return aScales;
};

/**
 * Reload legend with print box params
 * @returns {undefined}
 * @export
 */
nsVmap.nsToolsManager.Print.prototype.printController.prototype.reloadLegend = function () {
    oVmap.log('nsVmap.nsToolsManager.Print.prototype.printController.prototype.reloadLegend');

    if (goog.isDefAndNotNull(this['selectedModel']) && this['selectedModel']['format'] !== 'A0') {
        var compareDataPrintBox = null;
        if (this.$scope_["$root"]["compare_enabled"] && !this.printBoxCompare_.isHide()) {
            compareDataPrintBox = {
                extent: this.printBoxCompare_.getExtent(),
                size: this['useScaleFromList'] === true ? this['printedCompareMapSize_']['realSize'] : this.printBoxCompare_.getSize()
            }

            if (this.$scope_["manageComparePrintZoneInProgress"]) {
                compareDataPrintBox = false;
            }
        }

        var dataPrintBox = {
            extent: this.printBox_.getExtent(),
            size: this['useScaleFromList'] === true ? this['printedMapSize_']['realSize'] : this.printBox_.getSize()
        };
        if (this.$scope_["managePrintZoneInProgress"]) {
            dataPrintBox = false;
        }

        oVmap.getMapManager().getMapLegendTool().loadLegend(dataPrintBox, compareDataPrintBox);
    }
    
};

// Définit la directive et le controller
oVmap.module.directive('appPrint', nsVmap.nsToolsManager.Print.prototype.printDirective);
oVmap.module.controller('AppprintController', nsVmap.nsToolsManager.Print.prototype.printController);
