/* global goog, nsVmap, oVmap, ol, angular, scope, bootbox, vitisApp, destructor_form */

/**
 * @author: Armand Bahi
 * @Description: Fichier contenant la classe nsVmap.nsToolsManager.BasicSelect
 * cette classe permet l'interrogation de couches
 */
goog.provide('nsVmap.nsToolsManager.BasicSelect');

goog.require('oVmap');

/**
 * @classdesc
 * Class {@link nsVmap.nsToolsManager.BasicSelect}: allow select
 *
 * @constructor
 * @export
 */
nsVmap.nsToolsManager.BasicSelect = function () {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect');
};

/**
 * App-specific directive wrapping the select tools.
 * @return {angular.Directive} The directive specs.
 * @constructor
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectDirective = function () {
    oVmap.log("nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectDirective");
    return {
        restrict: 'A',
        scope: {
            'map': '=appMap',
            'lang': '=appLang',
            'currentAction': '=appAction'
        },
        controller: 'AppBasicselectController',
        controllerAs: 'ctrl',
        bindToController: true,
        templateUrl: oVmap['properties']['vmap_folder'] + '/' + 'template/tools/basicselect.html'
    };
};

/**
 * @ngInject
 * @constructor
 * @param {object} $scope
 * @param {object} $timeout
 * @param {object} $q
 * @returns {nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController}
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController = function ($scope, $timeout, $q, $translate) {
    oVmap.log("nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController");

    var this_ = this;

    /**
     * The vMap scope
     */
    $scope['vmapScope'] = oVmap['scope'];

    /**
     * @private
     */
    this.$scope_ = $scope;

    /**
     * @private
     */
    this.$timeout_ = $timeout;

    /**
     * @private
     */
    this.$q_ = $q;

    /**
     * @private
     */
    this.$translate_ = $translate;

    /**
     * Dummy canceler (use canceler.resolve() for cancel the ajax request)
     * @private
     */
    this.canceler_ = $q.defer();

    /**
     * Parent element
     */
    this.oSelect = oVmap.getToolsManager().getBasicTools().getSelect();

    /**
     * The current properties
     */
    this['properties'] = oVmap['properties'];

    /**
     * The current token
     */
    this['token'] = oVmap['properties']['token'];

    /**
     * Maximum popup mode objects
     * @type integer
     */
    this.limitPopup_ = oVmap['properties']["selection"]["limit_popup"];

    /**
     * Maximum list mode objects
     * @type integer
     */
    this['limitList_'] = oVmap['properties']["selection"]["limit_list"];

    /**
     * @private
     */
    this['allQueryables_'] = false;

    /**
     * contains the current selections
     * @type {array}
     */
    this['aSelections'] = [];

    /**
     * contains the selection before
     * @type {array}
     */
    this.aLastSelections = [];

    /**
     * Contains the information used by the table
     * @type object
     */
    this['tableSelection'] = {};

    /**
     * The selected business object
     */
    this['sSelectedBo'] = '';

    /**
     * Queryable business objects
     */
    this['oQueryableBOs'] = {};

    /**
     * WMS queryable layers
     */
    this['aQueryableLayers'] = {};

    /**
     * WMS GetFeatureInfo tree
     */
    this['aGetFeatureInfoTree'] = {};

    /**
     * Queryable business objects list
     */
    this['aBusinessObjectsList'] = [];

    // Rempli les bo disponibles
    oVmap['scope'].$on('layersChanged', function () {
        setTimeout(function () {
            this_.setQueryableObjects();
        });
    });
    setTimeout(function () {
        this_.setQueryableObjects();
    });

    // Effectue une sélection lors d'un click sur la carte
    this['map'].on('click', function (evt) {
        if (oVmap.getMap().getCurrentAction() === '') {

            // Modale GetFeatureInfo déjà affichée ?
            var bGFIModalOpened = angular.copy($('#getFeatureInfo-result-window').is(":visible"));

            // Re-crée la géométrie en WKT
            var olPoint = new ol.geom.Point(evt.coordinate);
            var EWKTGeometry = oVmap.getEWKTFromGeom(olPoint);

            // Ferme les outils
            oVmap.getToolsManager().getBasicTools().toggleOutTools();

            // Interroge les couches selectionnées
            this.queryByEWKTGeom(EWKTGeometry, olPoint);

            // Interroge les couches sélectionnées par GetFeatureInfo
            if (this['sSelectedBo'] === 'vmap_select_getfeatureinfo') {
                this.queryGetFeatureInfo(evt.coordinate, bGFIModalOpened)
            }
        }
    }, this);

    // Anulle l'édition quand on change d'outil
    oVmap['scope'].$on('toggleOutTools', function () {
        this_.canceler_.resolve();
    });

    // Lors du click sur 'echap', annule les requêtes en cours
    $(document).keydown(function (e) {
        if (e.keyCode === 27) {
            this_.canceler_.resolve();
        }
    });

    // Vide les popups quand on change de carte
    oVmap['scope'].$on('mapChanged', function () {
        this_.removePopups();
    });

    // Événement pour dire à l'appli que this_['sSelectedBo'] a changé
    $scope.$watch('ctrl.sSelectedBo', function (oldVal, newVal) {
        if (!goog.isString(oldVal) || !goog.isString(newVal)) {
            return;
        }
        if (!oldVal.length > 0 || !newVal.length > 0) {
            return;
        }
        if (oldVal === newVal) {
            return;
        }
        oVmap['scope'].$broadcast('selectedBoChanged');
    });

    this['dataFormPrintReport'] = {
        printReport: {},
        printReportStyle: null,
        selection: []
    };
    this['avaliablePrintReports'] = [];
    this['userPrintStyles'] = null;
};


// Interrogation par intersection géométrique

/**
 * Call sendQueryByEWKTGeom_ for the selected layers
 * @param {string} EWKTGeometry
 * @param {boolean} useTableFormat true to result in table format, false for popup
 * @param {string|undefined} sQueryBo Business object to query, this.sSelectedBo if undefined
 * @returns {undefined}
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.queryByEWKTGeom = function (EWKTGeometry, olPoint, sQueryBo) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.queryByEWKTGeom');

    var this_ = this;

    // Stoque les paramètres d'interrogation
    this.EWKTGeometry = EWKTGeometry;
    this.olPoint = olPoint;

    $('body').css({"cursor": "wait"});

    // Annule les anciennes requêtes
    this.canceler_.resolve();

    // Réinitialise canceler_
    this.canceler_ = this.$q_.defer();

    // Objet métier sur lequel effectuer la selection
    sQueryBo = goog.isDefAndNotNull(sQueryBo) ? sQueryBo : this['sSelectedBo'];

    // Interroge l'objet métier sélectionné
    if (goog.isDefAndNotNull(this['oQueryableBOs'][sQueryBo])) {

        // Ajoute les filtres des couches associées
        var sLayersFilter = this.getBOLayersFilters_(this['oQueryableBOs'][sQueryBo]);

        this.oSelect.queryByEWKTGeom({
            bo_id: sQueryBo,
            sFilter: sLayersFilter,
            EWKTGeometry: EWKTGeometry,
            canceler: this.canceler_,
            useTableFormat: false,
            buffer: this['oQueryableBOs'][sQueryBo]['bo_selection_buffer'],
            callback: function (data) {
                $('body').css({"cursor": ""});
                this_.replaceSelectionPopup(data, olPoint);
            }
        });
    } else {
        $('body').css({"cursor": ""});
    }
};

/**
 * Get the business object associated layers filter as a API usable string
 * @param {object} oQueryBo
 * @returns {String}
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.getBOLayersFilters_ = function (oQueryBo) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototypeBO.getLayersFilters_');
    var formReaderSrvc = angular.element(vitisApp.appMainDrtv).injector().get(["formReaderService"]);

    var oFilterValues = {};
    var oFilterDefinition = {};
    var sFilter = "";
    var filter = {
        'relation': 'AND',
        'operators': []
    };

    var oLayerFilter_;
    if (goog.isArray(oQueryBo['layers'])) {
        for (var i = 0; i < oQueryBo['layers'].length; i++) {
            if (oQueryBo['layers'][i].get('is_bo_filtered')) {
                oLayerFilter_ = oQueryBo['layers'][i].get('filter_values_cleared');
                if (goog.isObject(oLayerFilter_)) {
                    goog.object.extend(oFilterValues, oLayerFilter_);
                    oLayerFilterDef_ = oQueryBo['layers'][i].get('filter_form');
                    if(goog.isObject(oLayerFilterDef_)){
                        for (var field in oLayerFilter_){
                            var oFieldDef = formReaderSrvc["getFormElementDefinition"](field, 'search', oLayerFilterDef_);
                            oFilterDefinition[field] = oFieldDef;
                        }
                    }
                }
            }
        }
    }

    for (var key in oFilterValues) {

        if (oFilterValues[key] === '') {
            continue;
        }

        // Tableaux (utilisés pour le IN)
        if (goog.isArray(oFilterValues[key])) {

            // Supprime les valeurs vides
            for (var i = oFilterValues[key].length - 1; i >= 0; i--) {
                if(oFilterValues[key][i] === ""){
                    oFilterValues[key].splice(i, 1);
                }
            }

            // Supprime les tableaux vides
            if (oFilterValues[key].length === 0) {
                continue;
            }
        }
        var sValue = oFilterValues[key]
        //surcharge le filtre si une définition est rpésente dans le formulaire
        if(goog.isDefAndNotNull(oFilterDefinition[key]["comparator"])){
            var aOperators = [];
            for (var j = 0;j < oFilterDefinition[key]["comparator"].length;j++){
                var oOperator = {};
                if (goog.isDefAndNotNull(oFilterDefinition[key]["comparator"][j]["formater"])) {
                    sValue = oFilterDefinition[key]["comparator"][j]["formater"].replace(/<VALUE_TO_REPLACE>/g, sValue);
                }
                if(goog.isArray(oFilterDefinition[key]["comparator"][j]["compare_operator"])) {
                    oOperator["relation"] = (goog.isDefAndNotNull(oFilterDefinition[key]["comparator"][j]["relation"]) ? oFilterDefinition[key]["comparator"][j]["relation"] : "AND");
                    oOperator["operators"] = [];
                    for (var k = 0; k < oFilterDefinition[key]["comparator"][j]["compare_operator"].length; k++){
                        oOperator["operators"].push({
                            "column": (goog.isArray(oFilterDefinition[key]["comparator"][j]["column"]) ? oFilterDefinition[key]["comparator"][j]["column"][k] : key),
                            "compare_operator": oFilterDefinition[key]["comparator"][j]["compare_operator"][k],
                            "value": sValue
                        });
                    }
                }else{
                    oOperator["column"] = (goog.isDefAndNotNull(oFilterDefinition[key]["comparator"][j]["column"]) ? oFilterDefinition[key]["comparator"][j]["column"] : key);
                    oOperator["compare_operator"] = oFilterDefinition[key]["comparator"][j]["compare_operator"];
                    oOperator["value"] = sValue;
                }
                if(goog.isDefAndNotNull(oFilterDefinition[key]["comparator"][j]["compare_operator_options"])){
                    oOperator["compare_operator_options"] = oFilterDefinition[key]["comparator"][j]["compare_operator_options"];
                }
                aOperators.push(oOperator);
            }
            if(goog.isDefAndNotNull(oFilterDefinition[key]["comparator_relation"])){
                filter['operators'].push({
                    "relation": oFilterDefinition[key]["comparator_relation"],
                    "operators": aOperators
                });
            }else{
                filter['operators'] = filter['operators'].concat(aOperators);
            }
        } else {
            filter['operators'].push({
                'column': key,
                'compare_operator': goog.isArray(oFilterValues[key]) ? 'IN' : '=',
                'value': sValue
            });
        }
    } 
    sFilter = JSON.stringify(filter);

    return sFilter;
};


// Interrogation GetFeatureInfo

/**
 * queryGetFeatureInfo function - Lance une requête GetFeatureInfo sur les couches/calcques sélectionnés
 *
 * @param  {array} aCoordinate Coordonnées où a été fait le clic
 * @param  {boolean} bGFIModalOpened Détermine si la modale GetFeatureInfo est déjà ouverte
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.queryGetFeatureInfo = function (aCoordinate, bGFIModalOpened){

    var this_ = this;
    var requestSendCounter = 0;
    var requestResultCounter = 0;

    this['aGetFeatureInfoTree'] = this.getGFITree_(aCoordinate);

    for (var i = 0; i < this['aGetFeatureInfoTree'].length; i++) {
        if (goog.isDefAndNotNull(this['aGetFeatureInfoTree'])) {
            if (goog.isDefAndNotNull(this['aGetFeatureInfoTree'][i]['aSubLayers'])) {
                for (var ii = 0; ii < this['aGetFeatureInfoTree'][i]['aSubLayers'].length; ii++) {
                    if (goog.isDefAndNotNull(this['aGetFeatureInfoTree'][i]['aSubLayers'][ii]['url'])) {

                        // Passe l'état en cours
                        this['aGetFeatureInfoTree'][i]['aSubLayers'][ii]['state'] = 'loading';

                        // Incrémente le compteur de requêtes parties
                        requestSendCounter++;

                        // Lance la requête
                        ajaxRequest({
                            'method': 'GET',
                            'url': this['aGetFeatureInfoTree'][i]['aSubLayers'][ii]['url'],
                            'headers': {
                                'Accept': 'text/html'
                            },
                            'responseType': 'text',
                            'ajaxLoader': true,
                            'success': angular.bind(this, function (i, ii, response) {

                                // Incrémente le compteur de requêtes recues
                                requestResultCounter++;

                                this['aGetFeatureInfoTree'][i]['aSubLayers'][ii]['state'] = 'loaded';
                                if (goog.isString(response['data'])) {
                                    if (response['data'].length > 0) {
                                        this['aGetFeatureInfoTree'][i]['aSubLayers'][ii]['result'] = response['data'];
                                        this['aGetFeatureInfoTree'][i]['have_results'] = true;
                                    }
                                }

                                // Affiche la fenêtre resultante
                                if (requestSendCounter === requestResultCounter) {
                                    this.oSelect.showGetFeatureInfoResultWindow(angular.copy(this['aGetFeatureInfoTree']), aCoordinate, bGFIModalOpened);
                                }

                            }, i, ii),
                            'error': function(){

                                // Incrémente le compteur de requêtes recues
                                requestResultCounter++;

                                // Affiche la fenêtre resultante
                                if (requestSendCounter === requestResultCounter) {
                                    this.oSelect.showGetFeatureInfoResultWindow(angular.copy(this['aGetFeatureInfoTree']), aCoordinate, bGFIModalOpened);
                                }
                            }
                        });
                    }
                }
            }
        }
    }
}

/**
 * getGFITree_ function - Retourne un arbre contenant les différents calques/couches avec les URL utilisés pour la sélection GetFeatureInfo
 *
 * @param  {array} aCoordinate Coordonnées où a été fait le clic
 * @return {object}
 * @private
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.getGFITree_ = function (aCoordinate){

    var iViewResolution = this['map'].getView().getResolution();
    var sProjection = this['map'].getView().getProjection().getCode();
    var aGFITree = [];
    var aSubLayers, oLayerNode;

    for (var i = 0; i < this['aQueryableLayers'].length; i++) {
        if (goog.isArray(this['aQueryableLayers'][i].get('queryableSublayers'))) {
            aSubLayers = this['aQueryableLayers'][i].get('queryableSublayers');
            if (aSubLayers.length > 0) {
                oLayerNode = {
                    'name': this['aQueryableLayers'][i].get('name'),
                    'aSubLayers': []
                }
                for (var ii = 0; ii < aSubLayers.length; ii++) {
                    oLayerNode['aSubLayers'].push({
                        'name': aSubLayers[ii],
                        'url': this['aQueryableLayers'][i].getSource().getGetFeatureInfoUrl(aCoordinate, iViewResolution, sProjection, {'INFO_FORMAT': 'text/html', 'QUERY_LAYERS': aSubLayers[ii]})
                    });
                }
                aGFITree.push(oLayerNode);
            }
        }
    }

    return aGFITree;
}


// Ajout de la sélection

/**
 * Add the query result to this['aSelections'] removing the duplicate selections
 * @param {object} opt_options
 * @param {array<object>} opt_options.data
 * @param {string} opt_options.dataType summary|list|form
 * @param {function} opt_options.callback callback to run at the end of the function
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addQueryResult = function (opt_options) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addQueryResult');

    var this_ = this;
    var aData = goog.isDef(opt_options.data) ? opt_options.data : [];
    var dataType = goog.isDef(opt_options.dataType) ? opt_options.dataType : null;
    var callback = goog.isDef(opt_options.callback) ? opt_options.callback : function () {
        return 0;
    };

    this.oSelect.addQueryResult({
        data: aData,
        selection: this['aSelections'],
        dataType: dataType,
        callback: function (aSelections) {
            this_['aSelections'] = aSelections;
            callback.call();
        }
    });
};


// Fonctions mode popup

/**
 * Replace the last popup by a new
 * @param {array} aSelection
 * @param {ol.geom.Point|undefined} olPoint position where to center the popup
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.replaceSelectionPopup = function (aSelection, olPoint) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.replaceSelectionPopup');

    var this_ = this;

    // Vérifie que la feature soit bien présente
    for (var i = aSelection.length - 1; i >= 0; i--) {

        var error = false;
        if (!goog.isDefAndNotNull(aSelection[i]['bo_intersect_geom']) && !goog.isDefAndNotNull(aSelection[i]['olFeature'])) {
            error = true;
        }
        if (goog.isDefAndNotNull(aSelection[i]['olFeature'])) {
            if (!goog.isDefAndNotNull(aSelection[i]['olFeature'].getGeometry())) {
                error = true;
            }
        }
        if (error) {
            $.notify("L'objet sélectionné (" + aSelection[i]['bo_id_field'] + " = " + aSelection[i]['bo_id_value'] + ") n'a pas de géométrie");
            aSelection.splice(i, 1);
        }
    }

    if (aSelection.length === 0) {
        return;
    }

    // Garde en mémoire les anciennes selections pour supprimer leurs popup plus tard
    var newSelection = goog.array.clone(this['aSelections']);
    goog.array.extend(this.aLastSelections, newSelection);

    // Supprime toutes les anciennes popup
    this.removePopups();

    // Ajoute le résultat de dans this['aSelections']
    this_.addQueryResult({
        data: aSelection,
        dataType: 'summary',
        callback: function () {
            this_.closeSelectionPopup(this_.aLastSelections);
            this_.displaySelectionPopup(this_['aSelections'], olPoint);
        }
    });

};

/**
 * Display a selection of popups
 * @param {array} aSelection
 * @param {ol.geom.Point|undefined} olPoint position where to center the popup
 * @private
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.displaySelectionPopup = function (aSelection, olPoint) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.displaySelectionPopup');

    // Cas où un seul objet est selectionné
    if (aSelection.length === 1) {
        this.addSignleSelectionPopup_(aSelection[0], olPoint);
    }
    // Cas de multiple-selection
    else if (aSelection.length > 1) {
        this.addMultipleSelectionPopup_(aSelection, olPoint);
    }

    // Ajoute les résultats
    goog.array.insertArrayAt(this['aSelections'], aSelection, -1);

    // supprime les doublons
    this['aSelections'] = this.oSelect.removeDuplicateSelections(this['aSelections']);
};

/**
 * Crée une popup sur un unique enregistrement
 * @param  {object} oSelection
 * @return {object} oMapPopup
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addSignleSelectionPopup_ = function (oSelection, olPoint) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addSignleSelectionPopup_');

    // Supprime l'ancienne popup si besoin
    if (goog.isDef(oSelection['mapPopup'])) {
        oSelection['mapPopup'].remove(false);
    }

    // Si le résultat est un point, alors colle la popup sur le point
    if (goog.isDefAndNotNull(oSelection['olFeature'].getGeometry().getType)) {
        if (oSelection['olFeature'].getGeometry().getType() === 'Point') {
            olPoint = oSelection['olFeature'].getGeometry();
        }
    }

    // Crée la nouvelle popup
    oSelection['mapPopup'] = new nsVmap.Map.MapPopup(oSelection['olFeature'], olPoint);

    // Ajoute au bonton "close" la fonction qui rend la popup unlocked quand on la ferme
    this.addPopupCloselockEvent_(oSelection['mapPopup'], oSelection);

    // Ajoute le bouton "édition" si have_update_rights est true
    this.addPopupEditBtn_(oSelection['mapPopup'], oSelection);

    // Ajoute le bouton "fiche descriptive" si have_card_rights est true
    this.addPopupDisplayBtn_(oSelection['mapPopup'], oSelection);

    // Ajoute le bouton "Modifier la géométrie" si have_update_rights = true
    this.addPopupModifyGeomBtn_(oSelection['mapPopup'], oSelection);

    // Ajoute le bouton "Supprimer l'objet" si have_delete_rights = true
    this.addPopupDeleteToMenu_(oSelection['mapPopup'], oSelection);

    // Ajoute le bouton "Ajouter au panier"
    if (!oVmap['properties']['is_mobile']) {
        this.addPopupAddBasketMenu_(oSelection['mapPopup'], oSelection);
    }

    this.aActiveTools_ = oVmap['properties']['controls']['active_popup_tools'];

    if (this.aActiveTools_.indexOf('GoogleStreetView') !== -1) {
        // Google street view
        this.addPopupStreetViewMenu_(oSelection['mapPopup'], oSelection);
    }

    if (this.aActiveTools_.indexOf('Mapillary') !== -1) {
        // Mapillary
        this.addPopupMapillaryMenu_(oSelection['mapPopup'], oSelection);
    }

    if (this.aActiveTools_.indexOf('PrintReport') !== -1) {
        // Ajoute le bouton "télécharger un rapport" si have_update_rights = true
        this.addPopupPrintReportToMenu_(oSelection['mapPopup'], oSelection);
    }

    if (this.aActiveTools_.indexOf('ObjectLinkTo') !== -1) {
        // URL vers cet objet
        this.addPopupObjectLinkToMenu_(oSelection['mapPopup'], oSelection);
    }



    // Ajout du contennu
    oSelection['mapPopup'].displayMessage('<span class="vmap-popup-title">' + this.oSelect.parseFormTile(oSelection['bo_summarytitle'], oSelection['bo_summary']) + '</span>');
    oSelection['mapPopup'].addMessageTable(oSelection['bo_summary']);
    if (goog.isDef(oSelection['bo_image_path'])) {
        oSelection['mapPopup'].addPhoto(oSelection['bo_image_path']);
    }

    return oSelection['mapPopup'];
}

/**
 * Crée une popup à onglets sur de multiples enregistrements
 * @param  {object} oSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addMultipleSelectionPopup_ = function (aSelection, olPoint) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addMultipleSelectionPopup_');

    var this_ = this;
    var oTmpPopup;
    var sMultiPopupField;
    var aPopups = [];

    // Récupère sMultiSelectField
    if (goog.isDefAndNotNull(this['oQueryableBOs'])) {
        if (goog.isDefAndNotNull(this['sSelectedBo'])) {
            if (goog.isDefAndNotNull(this['oQueryableBOs'][this['sSelectedBo']])) {
                if (goog.isDefAndNotNull(this['oQueryableBOs'][this['sSelectedBo']]['bo_multipopup_field'])) {
                    sMultiPopupField = this['oQueryableBOs'][this['sSelectedBo']]['bo_multipopup_field'];
                }
                else {
                    sMultiPopupField = this['oQueryableBOs'][this['sSelectedBo']]['bo_id_field'];
                }
            }
        }
    }
    
    
    // Affiche toutes les popups
    for (var i = 0; i < aSelection.length; i++) {
        oTmpPopup = this.addSignleSelectionPopup_(aSelection[i], olPoint);
        oTmpPopup.sMultiPopupField = sMultiPopupField;
        aPopups.push(oTmpPopup);
    }

    if (!oVmap['properties']['is_mobile']){
        this.aActiveTools_ = oVmap['properties']['controls']['active_popup_tools'];

        // Renseigne à chacune des popup la liste des popups actives
        for (var i = 0; i < aPopups.length; i++) {
            var aMenus = [{
                'content': '<span class="icon-shopping_basket"></span> ' + this.$translate_.instant('VMAP_BASICSELECT_ADD_BASKET'),
                'event': function () {
                    this_.multipleSelectionPopupAddToBasket_(aSelection);
                }
            }]

            if (this.aActiveTools_.indexOf('GoogleStreetView') !== -1) {
                aMenus.push({
                    'content': '<span class="icon-street-view"></span> ' + this.$translate_.instant('VMAP_BASICSELECT_GOOGLE_STREET_VIEW'),
                    'event': function () {
                        this_.multipleSelectionPopupOpenStreetView_(aSelection);
                    }
                });
            }
            if (this.aActiveTools_.indexOf('Mapillary') !== -1) {
                aMenus.push({
                    'content': '<span class="icon-street-view"></span> ' + this.$translate_.instant('VMAP_BASICSELECT_MAPILLARY'),
                    'event': function () {
                        this_.multipleSelectionPopupMapillary_(aSelection);
                    }
                });
            }
            if (this.aActiveTools_.indexOf('PrintReport') !== -1) {
                aMenus.push({
                    'content': '<span class="icon-files-o"></span> ' + this.$translate_.instant('VMAP_BASICSELECT_PRINT_REPORTS_BTN'),
                    'event': function () {
                        this_.openModalPrintReport(aSelection);
                    }
                });
            }
            if (this.aActiveTools_.indexOf('ObjectLinkTo') !== -1) {
                aMenus.push({
                    'content': '<span class="icon-link_url"></span> ' + this.$translate_.instant('VMAP_BASICSELECT_LINK_TO_OBJECTS'),
                    'event': function () {
                        this_.multipleSelectionPopupGenerateLink_(aSelection);
                    }
                });
            }
            aPopups[i].setPopupsList(aPopups, aMenus);
        }

    } else {
        // Renseigne à chacune des popup la liste des popups actives
        for (var i = 0; i < aPopups.length; i++) {
        aPopups[i].setPreNextButtons(aPopups);
        }
    }

    // Définit toutes les popups inactives sauf la première
    for (var i = 0; i < aPopups.length; i++) {
        if (i > 0) {
            aPopups[i].setActive(false);
        }
    }
}

/**
 * Ajoute au bonton "close" la fonction qui rend la popup locked quand on la ferme
 * @param  {object} oPopup
 * @param  {object} oSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupCloselockEvent_ = function (oPopup, oSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupCloselockEvent_');

    oPopup.HTMLCloser_.selection = oSelection;
    oPopup.HTMLCloser_.addEventListener('click', function () {
        this.selection['locked'] = false;
    }, true);
}

/**
 * Ajoute le bouton "édition" si have_update_rights est true
 * @param  {object} oPopup
 * @param  {object} oSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupEditBtn_ = function (oPopup, oSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupEditBtn_');

    var scope = this.$scope_;

    if (oSelection['have_update_rights'] === "true" || oSelection['have_update_rights'] === true) {
        oPopup.HTMLEdit_ = document.createElement('a');
        oPopup.HTMLEdit_.href = '#';
        oPopup.HTMLEdit_.className = 'ol-popup-edit icon-draw';
        oPopup.HTMLButtonsContainer_.appendChild(oPopup.HTMLEdit_);

        // Ajoute l'interraction sur le bouton "édition"
        oPopup.HTMLEdit_.scope = this.$scope_;
        oPopup.HTMLEdit_.selection = oSelection;
        oPopup.HTMLEdit_.addEventListener('click', function () {
            var selection = this.selection;
            scope.$apply(function () {
                scope['ctrl'].displayEditFrom(selection);
            });
        }, true);
    }
}

/**
 * Ajoute le bouton "diche descriptive" si have_card_rights est true
 * @param  {object} oPopup
 * @param  {object} oSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupDisplayBtn_ = function (oPopup, oSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupDisplayBtn_');

    var scope = this.$scope_;

    if (oSelection['have_card_rights'] === "true" || oSelection['have_card_rights'] === true) {
        oPopup.HTMLCard_ = document.createElement('a');
        oPopup.HTMLCard_.href = '#';
        oPopup.HTMLCard_.className = 'ol-popup-card glyphicon glyphicon-file';
        oPopup.HTMLButtonsContainer_.appendChild(oPopup.HTMLCard_);

        // Ajoute l'interraction sur le bouton "fiche descriptive"
        oPopup.HTMLCard_.scope = this.$scope_;
        oPopup.HTMLCard_.selection = oSelection;
        oPopup.HTMLCard_.addEventListener('click', function () {
            var selection = this.selection;
            scope.$apply(function () {
                scope['ctrl'].displayObjectCard(selection);
            });
        }, true);
    }
}

/**
 * Ajoute le bouton "Modifier la géométrie" si have_update_rights = true
 * @param  {object} oPopup
 * @param  {object} oSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupModifyGeomBtn_ = function (oPopup, oSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupModifyGeomBtn_');

    var scope = this.$scope_;

    if ((oSelection['have_update_rights'] === "true" || oSelection['have_update_rights'] === true) &&
        (oSelection['enable_geom_updater'] === "true" || oSelection['enable_geom_updater'] === true)) {
        oPopup.HTMLEdit_ = document.createElement('a');
        oPopup.HTMLEdit_.href = '#';
        oPopup.HTMLEdit_.className = 'ol-popup-modify-geom icon-polygon';
        oPopup.HTMLButtonsContainer_.appendChild(oPopup.HTMLEdit_);

        // Ajoute l'interraction sur le bouton "Modifier la géométrie"
        oPopup.HTMLEdit_.scope = this.$scope_;
        oPopup.HTMLEdit_.selection = oSelection;
        oPopup.HTMLEdit_.addEventListener('click', function () {
            var selection = this.selection;
            scope.$apply(function () {
                scope['ctrl'].editFeature(selection);
            });
        }, true);
    }
}

/**
 * Ajoute le bouton "télécharger un rapport"
 * @param  {object} oPopup
 * @param  {object} oSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupPrintReportToMenu_ = function (oPopup, oSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupPrintReportToMenu_');

    var scope = this.$scope_;

    if (oSelection['have_delete_rights'] === "true" || oSelection['have_delete_rights'] === true) {
        oPopup.addAction({
            'content': '<span class="icon-file-o"></span> ' + this.$translate_.instant('VMAP_BASICSELECT_PRINT_REPORT_BTN'),
            'event': function () {
                scope.$apply(function () {
                    scope['ctrl'].openModalPrintReport([oSelection]);
                });
            }
        });
    }
}

/**
 * Ajoute le bouton "Supprimer l'objet" si have_delete_rights = true
 * @param  {object} oPopup
 * @param  {object} oSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupDeleteToMenu_ = function (oPopup, oSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupDeleteToMenu_');

    var scope = this.$scope_;

    if (oSelection['have_delete_rights'] === "true" || oSelection['have_delete_rights'] === true) {
        oPopup.addAction({
            'content': '<span class="icon-trash"></span> ' + this.$translate_.instant('VMAP_BASICSELECT_DELETE_OBJECT'),
            'event': function () {
                scope.$apply(function () {
                    scope['ctrl'].deleteObject(oSelection);
                });
            }
        });
    }
}

/**
 * Ajoute le bouton "Ajouter au panier"
 * @param  {object} oPopup
 * @param  {object} oSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupAddBasketMenu_ = function (oPopup, oSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupAddBasketMenu_');

    var scope = this.$scope_;

    oPopup.addAction({
        'content': '<span class="icon-shopping_basket"></span> ' + this.$translate_.instant('VMAP_BASICSELECT_ADD_BASKET'),
        'event': function () {
            scope.$apply(function () {
                scope['ctrl'].addToCard(oSelection);
            });
        }
    });
}

/**
 * Ajoute le bouton "Google street view"
 * @param  {object} oPopup
 * @param  {object} oSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupStreetViewMenu_ = function (oPopup, oSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupStreetViewMenu_');

    if (oVmap['properties']['selection']['street_view'] !== false) {
        var geometry = oSelection['olFeature'].getGeometry();
        var pointCoords = geometry.getFirstCoordinate();
        var proj = this['map'].getView().getProjection().getCode();
        pointCoords = ol.proj.transform(pointCoords, proj, 'EPSG:4326');
        var url = 'http://maps.google.com/maps?q=&layer=c&cbll=' + pointCoords[1] + ',' + pointCoords[0] + '';
        oPopup.addAction({
            'content': '<span class="icon-street-view"></span> ' + this.$translate_.instant('VMAP_BASICSELECT_GOOGLE_STREET_VIEW'),
            'event': function () {
                var win = window.open(url, '_blank');
                win.focus();
            }
        });
    }
}

/**
 * Ajoute le bouton "Mapillary"
 * @param  {object} oPopup
 * @param  {object} oSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupMapillaryMenu_ = function (oPopup, oSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupMapillaryMenu_');

    if (oVmap['properties']['selection']['street_view'] !== false) {
        var geometry = oSelection['olFeature'].getGeometry();
        var pointCoords = geometry.getFirstCoordinate();
        var proj = this['map'].getView().getProjection().getCode();
        pointCoords = ol.proj.transform(pointCoords, proj, 'EPSG:4326');

        var url = 'https://www.mapillary.com/app/?lat=' + pointCoords[1] + '&lng=' + pointCoords[0] + '&z=19';
        oPopup.addAction({
            'content': '<span class="icon-street-view"></span> ' + this.$translate_.instant('VMAP_BASICSELECT_MAPILLARY'),
            'event': function () {
                var win = window.open(url, '_blank');
                win.focus();
            }
        });
    }
}



/**
 * Ajoute le bouton "URL vers cet objet"
 * @param  {object} oPopup
 * @param  {object} oSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupObjectLinkToMenu_ = function (oPopup, oSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addPopupObjectLinkToMenu_');

    var this_ = this;
    var scope = this.$scope_;

    oPopup.addAction({
        'content': '<span class="icon-link_url"></span> ' + this.$translate_.instant('VMAP_BASICSELECT_LINK_TO_OBJECT'),
        'event': function () {
            scope.$apply(function () {

                // generate url for this object;
                var sUrl = this_['properties']['web_server_name'] + "/" + this_['properties']['application'];

                if (goog.isDefAndNotNull(this_['properties']['environment'])) {
                    if (this_['properties']['environment'] != "") {
                        sUrl += "_" + this_['properties']['environment'];
                    }
                }

                sUrl += "?mode_id=vmap&";

                var oMapCatalog = oVmap.getMapManager().getMapCatalog();
                var iMapId = oMapCatalog["maps"][oMapCatalog["usedMap"]]["map_id"];

                sUrl += "map_id=" + iMapId + "&";
                sUrl += "bo_id=" + oSelection["bo_type"] + "&";
                sUrl += "ids=" + oSelection["bo_id_value"];

                //copy url to clipboard
                var textArea = document.createElement("textarea");
                textArea.value = sUrl;
                document.body.appendChild(textArea);
                textArea.focus();
                textArea.select();

                try {
                    var successful = document.execCommand('copy');
                    if (successful) {
                        $.notify(this_.$translate_.instant('VMAP_BASICSELECT_LINK_TO_OBJECT_MESSAGE') + ' : \n' + sUrl, 'success');
                    } else {
                        $.notify(this_.$translate_.instant('VMAP_BASICSELECT_LINK_TO_OBJECT_MESSAGE_ERROR') + ' : \n' + sUrl, 'error');
                    }
                } catch (err) {
                    $.notify(this_.$translate_.instant('VMAP_BASICSELECT_LINK_TO_OBJECT_MESSAGE_ERROR') + ' : \n' + sUrl, 'error');
                    console.error('Fallback: Oops, unable to copy', err);
                }
                document.body.removeChild(textArea);
            });
        }
    });
}

/**
 * Événement appelé depuis la popup multiple
 * Permet d'ajouter les éléments au panier
 * @param {array} aSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.multipleSelectionPopupAddToBasket_ = function (aSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.multipleSelectionPopupAddToBasket_');

    if (!goog.isArray(aSelection)) {
        console.error('selection not valid');
        return false;
    }
    if (!aSelection.length > 0) {
        console.error('selection not valid');
        return false;
    }

    aUnlockedSelections = [];
    for (var i = 0; i < aSelection.length; i++) {
        if(aSelection[i]['locked'] !== false){
            aUnlockedSelections.push(aSelection[i]);
        }
    }

    this.oSelect.addToCard(aUnlockedSelections, false, false);
}

/**
 * Événement appelé depuis la popup multiple
 * Permet d'ouvrir google street view
 * @param {array} aSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.multipleSelectionPopupOpenStreetView_ = function (aSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.multipleSelectionPopupOpenStreetView_');

    if (!goog.isArray(aSelection)) {
        console.error('selection not valid');
        return false;
    }
    if (!aSelection.length > 0) {
        console.error('selection not valid');
        return false;
    }

    aUnlockedSelections = [];
    for (var i = 0; i < aSelection.length; i++) {
        if (aSelection[i]['locked'] !== false) {
            aUnlockedSelections.push(aSelection[i]);
        }
    }

    var sIntersectGeom = aUnlockedSelections[0]['bo_intersect_geom'];
    var oIntersectGeom = oVmap.getGeomFromEWKT(sIntersectGeom, 'EPSG:4326');
    var pointCoords = oIntersectGeom.getFirstCoordinate();
    var url = 'http://maps.google.com/maps?q=&layer=c&cbll=' + pointCoords[1] + ',' + pointCoords[0] + '';
    var win = window.open(url, '_blank');
    win.focus();
}

/**
 * Événement appelé depuis la popup multiple
 * Permet d'ouvrir mapillary
 * @param {array} aSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.multipleSelectionPopupMapillary_ = function (aSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.multipleSelectionPopupMapillary_');

    if (!goog.isArray(aSelection)) {
        console.error('selection not valid');
        return false;
    }
    if (!aSelection.length > 0) {
        console.error('selection not valid');
        return false;
    }

    aUnlockedSelections = [];
    for (var i = 0; i < aSelection.length; i++) {
        if (aSelection[i]['locked'] !== false) {
            aUnlockedSelections.push(aSelection[i]);
        }
    }

    var sIntersectGeom = aUnlockedSelections[0]['bo_intersect_geom'];
    var oIntersectGeom = oVmap.getGeomFromEWKT(sIntersectGeom, 'EPSG:4326');
    var pointCoords = oIntersectGeom.getFirstCoordinate();
    var url = 'https://www.mapillary.com/app/?lat=' + pointCoords[1] + '&lng=' + pointCoords[0] + '&z=19'
    var win = window.open(url, '_blank');
    win.focus();
}

/**
 * Événement appelé depuis la popup multiple
 * Permet de générer une URL lien vers ces objets
 * @param {array} aSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.multipleSelectionPopupGenerateLink_ = function (aSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.multipleSelectionPopupGenerateLink_');

    if (!goog.isArray(aSelection)) {
        console.error('selection not valid');
        return false;
    }
    if (!aSelection.length > 0) {
        console.error('selection not valid');
        return false;
    }

    aUnlockedSelections = [];
    for (var i = 0; i < aSelection.length; i++) {
        if (aSelection[i]['locked'] !== false) {
            aUnlockedSelections.push(aSelection[i]);
        }
    }

    // generate url for this object;
    var sUrl = this['properties']['web_server_name'] + "/" + this['properties']['application'];

    if (goog.isDefAndNotNull(this['properties']['environment'])) {
        if (this['properties']['environment'] != "") {
            sUrl += "_" + this['properties']['environment'];
        }
    }

    sUrl += "?mode_id=vmap&";

    var oMapCatalog = oVmap.getMapManager().getMapCatalog();
    var iMapId = oMapCatalog["maps"][oMapCatalog["usedMap"]]["map_id"];
    var sBoId = aUnlockedSelections[0]['bo_type'];
    var sIds = aUnlockedSelections[0]['bo_id_value'];
    for (var i = 1; i < aUnlockedSelections.length; i++) {
        sIds += '|' + aUnlockedSelections[i]['bo_id_value'];
    }

    sUrl += "map_id=" + iMapId + "&";
    sUrl += "bo_id=" + sBoId + "&";
    sUrl += "ids=" + sIds;

    //copy url to clipboard
    var textArea = document.createElement("textarea");
    textArea.value = sUrl;
    document.body.appendChild(textArea);
    textArea.focus();
    textArea.select();

    try {
        var successful = document.execCommand('copy');
        if (successful) {
            $.notify('L\'URL d\'accés à cette objet a été copié dans votre presse-papier : \n' + sUrl, 'success');
        } else {
            $.notify('L\'URL d\'accés à cette objet n\'a pas pu être copié dans votre presse-papier : \n' + sUrl, 'error');
        }
    } catch (err) {
        $.notify('L\'URL d\'accés à cette objet n\'a pas pu être copié dans votre presse-papier : \n' + sUrl, 'error');
        console.error('Fallback: Oops, unable to copy', err);
    }
    document.body.removeChild(textArea);
}

/**
 * Center the map on a popup
 * @param {object} selection
 * @param {object} selection.olFeature
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.centerMapOnPopup = function (selection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.basicSelectController.centerMapOnPopup');

    if (!goog.isDefAndNotNull(selection['olFeature'])) {
        console.error("selection['olFeature'] undefined:", selection['olFeature'])
        return 0;
    }

    var geom = selection['olFeature'].getGeometry();

    if (!goog.isDefAndNotNull(geom)) {
        console.error("selection['olFeature'].getGeometry() not defined");
        return 0;
    }

    if (geom.getType() === 'Point') {
        this['map'].getView().setCenter(geom.getCoordinates());
    } else if (geom.getType() === 'MultiPoint') {
        if (geom.getPoints().length === 1) {
            this['map'].getView().setCenter(geom.getPoint(0).getCoordinates());
        } else {
            this['map'].getView().fit(geom);
        }
    } else {
        this['map'].getView().fit(geom);
    }

};


// Foncitons Édition/consultation formReader

/**
 * Display the object card
 * @param {object} selection
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.displayObjectCard = function (selection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.displayObjectCard');
    this.oSelect.displayObjectCard(selection);
};

/**
 * Display the edit form
 * @param {object} selection
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.displayEditFrom = function (selection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.displayEditFrom');
    var this_ = this;
    var sQueryBo = angular.copy(selection['bo_type']);
    this.oSelect.displayEditFrom(selection, function () {
        this_.queryByEWKTGeom(this_.EWKTGeometry, this_.olPoint, sQueryBo);
    });
};


// Édition de la géométrie


/**
 * Start the geometry edition
 * @param {object} selection
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.editFeature = function (selection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.editFeature');

    // Ferme l'ensemble des popups
    this.closeSelectionPopup();

    this.oSelect.editFeature(selection);
};


// Suppression

/**
 * Suggest if realy want to delete the object and call submitDeleteObject
 * @param {object} selection
 * @param {object} selection.bo_type
 * @param {object} selection.bo_id_value
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.deleteObject = function (selection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.basicSelectController.deleteObject');

    var this_ = this;

    this.oSelect.deleteObject(selection, function () {

        // Ferme les popup
        this_.closeSelectionPopup();

        // Supprime de la selection en cours
        this_.removeFromTableSelection(selection);
    });
};


// Utilitaires


/**
 * setQueryableBos function - Définit les Calques et objets métiers interrogeables
 *
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.setQueryableObjects = function () {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.setQueryableObjects');
    var this_ = this;

    this_.$scope_.$applyAsync(function () {

        // rempli this.oQueryableBOs et aQueryableBOs
        this_['aQueryableLayers'] = oVmap.getMapManager().getQueryableGetFeatureInfoLayers(true);
        this_['oQueryableBOs'] = oVmap.getMapManager().getQueryableBusinessObjects(true, true, false, true, false);
        this_['aQueryableBOs'] = oVmap.getMapManager().getQueryableBusinessObjectsAsArray(true, true, false, true, false);
        this_['aBusinessObjectsList'] = Object.keys(this_['oQueryableBOs']);

        // Élément sélectionné par défaut
        if (!goog.isDefAndNotNull(this_['oQueryableBOs'][this_['sSelectedBo']])) {

            var iSmallestIndex = 1000000;
            var sFirstBo = '';

            for (var bo in this_['oQueryableBOs']) {
                if (this_['oQueryableBOs'][bo]['bo_index'] < iSmallestIndex) {
                    iSmallestIndex = this_['oQueryableBOs'][bo]['bo_index'];
                    sFirstBo = this_['oQueryableBOs'][bo]['bo_id'];
                }
            }
            this_['sSelectedBo'] = sFirstBo;
        }
        if (oVmap['properties']['selection']['enable_getfeatureinfo'] === true) {
            if (goog.isArray(this_['aQueryableLayers'])) {
                if (this_['aQueryableLayers'].length > 0) {
                    this_['sSelectedBo'] = 'vmap_select_getfeatureinfo';
                }
            }
        }
    });
}

/**
 * Remove the popups and empty this['aSelections']
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.removePopups = function () {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.removePopups');

    // Supprime toutes les anciennes popup
    this.closeSelectionPopup();

    // Supprime toutes les anciennes selections
    this.removeSelections();
};

/**
 * Remove all the selections from this['aSelections']
 * @param {string|undefined} bo_id if defined, the function will only remove the selections witch belong the business object
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.removeSelections = function (bo_id) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.removeSelections');

    // Supprime toutes les selections
    for (var i = this['aSelections'].length - 1; i >= 0; i--) {

        // Si bo_id est défini, supprime uniquement les selections correspondantes au bo_id
        if (goog.isDefAndNotNull(bo_id)) {
            if (this['aSelections'][i]['bo_type'] === bo_id)
                goog.array.removeAt(this['aSelections'], i);
        } else {
            goog.array.removeAt(this['aSelections'], i);
        }
    }
};

/**
 * Remove an element from this.aSelections
 * @param {type} selection
 * @returns {undefined}
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.removeFromTableSelection = function (selection) {

    for (var i = this['aSelections'].length - 1; i >= 0; i--) {
        if (angular.equals(this['aSelections'][i], selection)) {
            this['aSelections'].splice(i, 1);
        }
    }
};

/**
 * Remove the popups from aSelection or this['aSelections']
 * @param {array|undefined} aSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.closeSelectionPopup = function (aSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.closeSelectionPopup');

    aSelection = goog.isDefAndNotNull(aSelection) ? aSelection : this['aSelections'];

    for (var i = 0; i < aSelection.length; i++) {
        if (goog.isDef(aSelection[i]['mapPopup']))
            aSelection[i]['mapPopup'].remove();
    }

};

/**
 * Add the selected elements from the table to the card
 * @param {object} oSelection
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addToCard = function (oSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.tableAddToCard');
    this.oSelect.addToCard([oSelection], false, false);
};

/**
 * Open modal print report
 * @param {object} oSelection
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.openModalPrintReport = function (aSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.openModalPrintReport');
    if (!goog.isDef(aSelection) || aSelection.length <= 0) {
        console.error('aSelection empty or inexistent');
        return 0;
    }

    // Charge les styles disponibles pour l'utilisateur
    this.setUserPrintStyle();

    if (goog.isDefAndNotNull(this['sSelectedBo'])) {
        this.dataFormPrintReport.selection = aSelection;
        this.setAvaliablePrintReports(this['sSelectedBo']);
        $('#vmap-basicselect-print-report').modal('show');
    } else {
        $.notify('Erreur lors de la selection de l\'objet métier.', 'error');
    }
};

/**
 * Close modal print report
 * @param {object} oSelection
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.closeForm = function () {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.closeForm');
    this.dataFormPrintReport = {
        printReport: {},
        printReportStyle: null,
        selection: []
    };
    $('#vmap-basicselect-print-report').modal('hide');
};

/**
 * Set the avaliable business objects
 * @param {string} business_object_id
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.setAvaliablePrintReports = function (business_object_id) {
    oVmap.log('nsVmap.nsToolsManager.InfoContainer.infocontainerController.setAvaliablePrintReports');

    var this_ = this;

    // Vide le tableau de rapports en cours
    if (goog.isArray(this['avaliablePrintReports']))
        this['avaliablePrintReports'].length = 0;
    else
        this['avaliablePrintReports'] = [];

    if (!goog.isDefAndNotNull(business_object_id)) {
        $.notify('Aucun rapport disponible', 'warning');
        return 0;
    }

    // Recharge les nouvelles valeurs du tableau de rapports
    ajaxRequest({
        'method': 'GET',
        'url': oVmap['properties']['api_url'] + '/vmap/printreports',
        'headers': {
            'Accept': 'application/x-vm-json'
        },
        'params': {
            'attributs': 'name|printreport_id|business_object_id|business_object_id_field|multiobject|active_style',
            'filter': {
                "column": "business_object_id",
                "compare_operator": "=",
                "value": business_object_id
            }
        },
        'scope': this.$scope_,
        'success': function (response) {
            if (!goog.isDefAndNotNull(response['data'])) {
                $.notify('Erreur lors du chargement des rapports pour ' + business_object_id, 'error');
                console.error('response.data undefined: ', response);
                $('#vmap-basicselect-print-report').modal('hide');
                return 0;
            }
            if (!goog.isDefAndNotNull(response['data']['data'])) {
                $.notify('Aucun rapport disponible pour ' + business_object_id, 'warning');
                $('#vmap-basicselect-print-report').modal('hide');
                return 0;
            }
            this_['avaliablePrintReports'] = response['data']['data'];
        }
    });
};

/**
 * Set the user print style
 * @param {string} business_object_id
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.setUserPrintStyle = function () {
    oVmap.log('nsVmap.nsToolsManager.InfoContainer.infocontainerController.setUserPrintStyle');

    var this_ = this;

    // Ne charge les styles de l'utilisateur que la première fois
    if (!goog.isDefAndNotNull(this['userPrintStyles'])) {
        // Définit la variable pour ne plus recharger les styles
        this['userPrintStyles'] = [];
    
        // Recharge les nouvelles valeurs du tableau de rapports
        oVmap.getUserPrintStyle().then(function (userPrintStyles) {
            this_['userPrintStyles'] = userPrintStyles;
        });
    }

};

/**
 * Generate the report
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.generatePrintReport = function () {
    oVmap.log('nsVmap.nsToolsManager.InfoContainer.infocontainerController.generatePrintReport');

    var aSelection = this.dataFormPrintReport.selection;
    var oPrintReport = this.dataFormPrintReport.printReport;

    if (!goog.isArray(aSelection)) {
        $.notify('Impossible de récupérer la sélection');
        return 0;
    }
    if (aSelection.length <= 0) {
        $.notify('Aucun élément sélectionné');
        return 0;
    }
        
    // Construction de aIds
    var aIds = [];
    for (var i = 0; i < aSelection.length; i++) {
        if (goog.isDefAndNotNull(aSelection[i][oPrintReport['business_object_id_field']])) {
            aIds.push(aSelection[i][oPrintReport['business_object_id_field']]);
        } else if (oPrintReport['business_object_id_field'] == aSelection[i]['bo_id_field'] &&
            goog.isDefAndNotNull(aSelection[i]['bo_id_value'])) {
            aIds.push(aSelection[i]['bo_id_value']);
        }
    }

    if (!aIds.length > 0) {
        $.notify('Génération du document impossible');
        console.error('generated aIds is empty');
        return 0;
    }

    oVmap.generatePrintReport({
        'printReportId': oPrintReport['printreport_id'],
        'ids': aIds,
        'printReportStyleId': this.dataFormPrintReport.printReportStyle
    });

    this.closeForm();
};

// Définit la directive et le controller
oVmap.module.directive('appBasicselect', nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectDirective);
oVmap.module.controller('AppBasicselectController', nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController);
